package org.jeecg.modules.ctop.controller;

import cn.com.ctop.common.utils.HttpUtils;
import cn.com.ctop.common.utils.PropertiesUtils;
import cn.com.ctop.toutiao.common.BytedanceInterfaceConstant;
import com.alibaba.fastjson.JSONObject;
import com.google.gson.Gson;
import constant.KuaishouInterfaceConstant;
import org.apache.commons.lang.StringUtils;
import org.jeecg.modules.ctop.entity.CTopOauthToken;
import org.jeecg.modules.ctop.mapper.CTopOauthTokenMapper;
import org.jeecg.modules.kuaishou.service.IKuaishouInterfaceService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.util.HashMap;
import java.util.Map;
import java.util.UUID;

@Controller
@RequestMapping("/auth")
public class AuthController {
    private static final Logger logger = LoggerFactory.getLogger(AuthController.class);
    @Autowired
    private IKuaishouInterfaceService kuaishouInterfaceService;
    @RequestMapping("/begin")
    public String authBegin(HttpServletRequest request,
                            HttpServletResponse response,
                            @RequestParam("auth_type")String authType){
        try{
            String codeUrl = "";
            switch (authType)
            {
                case "bytedance":
                    codeUrl = this.getByteDanceCodeUrl(UUID.randomUUID().toString());
                    System.out.println(codeUrl);
                    return "redirect:"+codeUrl;
                case "kuaishou":
                    codeUrl = this.getKuaishouCodeUrl(UUID.randomUUID().toString());
                    System.out.println(codeUrl);
                    return "redirect:"+codeUrl;
                default:
                    System.out.println("");

            }
        }catch (Exception e){
            e.printStackTrace();
        }

        return "auth_error";
    }
    @RequestMapping("/bytedance")
    @ResponseBody
    public String bytedance(HttpServletRequest request,
                            HttpServletResponse response,
                            @RequestParam("auth_code")String authCode,
                            @RequestParam("state")String state,@RequestParam("accountId")String accountId) {
        System.out.println(request.getQueryString());
        if(StringUtils.isNotBlank(authCode)){
            System.out.println(this.getByteDanceAccessToken(authCode,accountId));
            return "auth_success";
        }
        return "auth_fail";
    }

    @GetMapping("bytedance/refreshtoken")


    public String getByteDanceAccessToken(String authCode, String accountId) {
        Map<String, Object> param = new HashMap<String, Object>();
        param.put("app_id",PropertiesUtils.getValue("bytedance_config","bytedance_appid"));
        param.put("secret",PropertiesUtils.getValue("bytedance_config","bytedance_secret"));
        param.put("grant_type","auth_code");
        param.put("auth_code",authCode);

        Gson gson = new Gson();
        String result = "";
        try{
            System.out.println(PropertiesUtils.getValue("bytedance_config","bytedance_api_url")+ BytedanceInterfaceConstant.AUTH_TOKEN);
            result = HttpUtils.httpPostRequest(PropertiesUtils.getValue("bytedance_config","bytedance_api_url")+BytedanceInterfaceConstant.AUTH_TOKEN,param,new HashMap<>());
            System.out.println(result);
            JSONObject resultObject = JSONObject.parseObject(result);
            Integer code = resultObject.getInteger("code");
            String message = resultObject.getString("message");
            if(null == code||code!=0){
                logger.info("获取token失败,accountId:{},message:{}",accountId,message);
            }
            JSONObject data = resultObject.getJSONObject("data");
            CTopOauthToken token = new CTopOauthToken(accountId,data);
            cTopOauthTokenMapper.insert(token);
        }catch (Exception e){
            e.printStackTrace();
        }
        return result;
    }

    @Autowired
    private CTopOauthTokenMapper cTopOauthTokenMapper;

    public String getKuaishouCodeUrl(String state) throws UnsupportedEncodingException {
        StringBuffer sb = new StringBuffer();
        sb.append(PropertiesUtils.getValue("kuaishou_config","kuaishou_auth_url"))
                .append(KuaishouInterfaceConstant.AUTH_PATH)
                .append("?app_id="+PropertiesUtils.getValue("kuaishou_config","kuaishou_appid"))
                .append("&state="+state)
                .append("&scope=%5B%22report_service%22%2C%22ad_query%22%2C%22ad_manage%22%5D")
                .append("&redirect_uri="+ URLEncoder.encode(PropertiesUtils.getValue("kuaishou_config","kuaishou_callback_url"),"UTF-8"));
        return sb.toString();
    }


    public String getByteDanceCodeUrl(String state) throws UnsupportedEncodingException {
        StringBuffer sb = new StringBuffer();
        sb.append(PropertiesUtils.getValue("bytedance_config","bytedance_auth_url"))
                .append(BytedanceInterfaceConstant.AUTH_PATH)
                .append("?app_id="+PropertiesUtils.getValue("bytedance_config","bytedance_appid"))
                .append("&state="+state)
                .append("&redirect_uri="+ URLEncoder.encode(PropertiesUtils.getValue("bytedance_config","bytedance_callback_url"),"UTF-8"));
        return sb.toString();
    }
}
