package org.jeecg.modules.ctop.controller;

import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.common.aspect.annotation.AutoLog;
import org.jeecg.common.util.oConvertUtils;
import org.jeecg.modules.ctop.entity.CTopOauthToken;
import org.jeecg.modules.ctop.service.ICTopOauthTokenService;
import java.util.Date;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import lombok.extern.slf4j.Slf4j;

import org.jeecgframework.poi.excel.ExcelImportUtil;
import org.jeecgframework.poi.excel.def.NormalExcelConstants;
import org.jeecgframework.poi.excel.entity.ExportParams;
import org.jeecgframework.poi.excel.entity.ImportParams;
import org.jeecgframework.poi.excel.view.JeecgEntityExcelView;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import org.springframework.web.servlet.ModelAndView;
import com.alibaba.fastjson.JSON;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

 /**
 * @Description: oauth授权码
 * @Author: jeecg-boot
 * @Date:   2019-07-09
 * @Version: V1.0
 */
@Slf4j
@Api(tags="oauth授权码")
@RestController
@RequestMapping("/ctop/cTopOauthToken")
public class CTopOauthTokenController {
	@Autowired
	private ICTopOauthTokenService cTopOauthTokenService;
	
	/**
	  * 分页列表查询
	 * @param cTopOauthToken
	 * @param pageNo
	 * @param pageSize
	 * @param req
	 * @return
	 */
	@AutoLog(value = "oauth授权码-分页列表查询")
	@ApiOperation(value="oauth授权码-分页列表查询", notes="oauth授权码-分页列表查询")
	@GetMapping(value = "/list")
	public Result<IPage<CTopOauthToken>> queryPageList(CTopOauthToken cTopOauthToken,
									  @RequestParam(name="pageNo", defaultValue="1") Integer pageNo,
									  @RequestParam(name="pageSize", defaultValue="10") Integer pageSize,
									  HttpServletRequest req) {
		Result<IPage<CTopOauthToken>> result = new Result<IPage<CTopOauthToken>>();
		QueryWrapper<CTopOauthToken> queryWrapper = QueryGenerator.initQueryWrapper(cTopOauthToken, req.getParameterMap());
		Page<CTopOauthToken> page = new Page<CTopOauthToken>(pageNo, pageSize);
		IPage<CTopOauthToken> pageList = cTopOauthTokenService.page(page, queryWrapper);
		result.setSuccess(true);
		result.setResult(pageList);
		return result;
	}
	
	/**
	  *   添加
	 * @param cTopOauthToken
	 * @return
	 */
	@AutoLog(value = "oauth授权码-添加")
	@ApiOperation(value="oauth授权码-添加", notes="oauth授权码-添加")
	@PostMapping(value = "/add")
	public Result<CTopOauthToken> add(@RequestBody CTopOauthToken cTopOauthToken) {
		Result<CTopOauthToken> result = new Result<CTopOauthToken>();
		try {
			cTopOauthTokenService.save(cTopOauthToken);
			result.success("添加成功！");
		} catch (Exception e) {
			log.error(e.getMessage(),e);
			result.error500("操作失败");
		}
		return result;
	}
	
	/**
	  *  编辑
	 * @param cTopOauthToken
	 * @return
	 */
	@AutoLog(value = "oauth授权码-编辑")
	@ApiOperation(value="oauth授权码-编辑", notes="oauth授权码-编辑")
	@PutMapping(value = "/edit")
	public Result<CTopOauthToken> edit(@RequestBody CTopOauthToken cTopOauthToken) {
		Result<CTopOauthToken> result = new Result<CTopOauthToken>();
		CTopOauthToken cTopOauthTokenEntity = cTopOauthTokenService.getById(cTopOauthToken.getId());
		if(cTopOauthTokenEntity==null) {
			result.error500("未找到对应实体");
		}else {
			boolean ok = cTopOauthTokenService.updateById(cTopOauthToken);
			//TODO 返回false说明什么？
			if(ok) {
				result.success("修改成功!");
			}
		}
		
		return result;
	}
	
	/**
	  *   通过id删除
	 * @param id
	 * @return
	 */
	@AutoLog(value = "oauth授权码-通过id删除")
	@ApiOperation(value="oauth授权码-通过id删除", notes="oauth授权码-通过id删除")
	@DeleteMapping(value = "/delete")
	public Result<?> delete(@RequestParam(name="id",required=true) String id) {
		try {
			cTopOauthTokenService.removeById(id);
		} catch (Exception e) {
			log.error("删除失败",e.getMessage());
			return Result.error("删除失败!");
		}
		return Result.ok("删除成功!");
	}
	
	/**
	  *  批量删除
	 * @param ids
	 * @return
	 */
	@AutoLog(value = "oauth授权码-批量删除")
	@ApiOperation(value="oauth授权码-批量删除", notes="oauth授权码-批量删除")
	@DeleteMapping(value = "/deleteBatch")
	public Result<CTopOauthToken> deleteBatch(@RequestParam(name="ids",required=true) String ids) {
		Result<CTopOauthToken> result = new Result<CTopOauthToken>();
		if(ids==null || "".equals(ids.trim())) {
			result.error500("参数不识别！");
		}else {
			this.cTopOauthTokenService.removeByIds(Arrays.asList(ids.split(",")));
			result.success("删除成功!");
		}
		return result;
	}
	
	/**
	  * 通过id查询
	 * @param id
	 * @return
	 */
	@AutoLog(value = "oauth授权码-通过id查询")
	@ApiOperation(value="oauth授权码-通过id查询", notes="oauth授权码-通过id查询")
	@GetMapping(value = "/queryById")
	public Result<CTopOauthToken> queryById(@RequestParam(name="id",required=true) String id) {
		Result<CTopOauthToken> result = new Result<CTopOauthToken>();
		CTopOauthToken cTopOauthToken = cTopOauthTokenService.getById(id);
		if(cTopOauthToken==null) {
			result.error500("未找到对应实体");
		}else {
			result.setResult(cTopOauthToken);
			result.setSuccess(true);
		}
		return result;
	}

  /**
      * 导出excel
   *
   * @param request
   * @param response
   */
  @RequestMapping(value = "/exportXls")
  public ModelAndView exportXls(HttpServletRequest request, HttpServletResponse response) {
      // Step.1 组装查询条件
      QueryWrapper<CTopOauthToken> queryWrapper = null;
      try {
          String paramsStr = request.getParameter("paramsStr");
          if (oConvertUtils.isNotEmpty(paramsStr)) {
              String deString = URLDecoder.decode(paramsStr, "UTF-8");
              CTopOauthToken cTopOauthToken = JSON.parseObject(deString, CTopOauthToken.class);
              queryWrapper = QueryGenerator.initQueryWrapper(cTopOauthToken, request.getParameterMap());
          }
      } catch (UnsupportedEncodingException e) {
          e.printStackTrace();
      }

      //Step.2 AutoPoi 导出Excel
      ModelAndView mv = new ModelAndView(new JeecgEntityExcelView());
      List<CTopOauthToken> pageList = cTopOauthTokenService.list(queryWrapper);
      //导出文件名称
      mv.addObject(NormalExcelConstants.FILE_NAME, "oauth授权码列表");
      mv.addObject(NormalExcelConstants.CLASS, CTopOauthToken.class);
      mv.addObject(NormalExcelConstants.PARAMS, new ExportParams("oauth授权码列表数据", "导出人:Jeecg", "导出信息"));
      mv.addObject(NormalExcelConstants.DATA_LIST, pageList);
      return mv;
  }

  /**
      * 通过excel导入数据
   *
   * @param request
   * @param response
   * @return
   */
  @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
  public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
      MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
      Map<String, MultipartFile> fileMap = multipartRequest.getFileMap();
      for (Map.Entry<String, MultipartFile> entity : fileMap.entrySet()) {
          MultipartFile file = entity.getValue();// 获取上传文件对象
          ImportParams params = new ImportParams();
          params.setTitleRows(2);
          params.setHeadRows(1);
          params.setNeedSave(true);
          try {
              List<CTopOauthToken> listCTopOauthTokens = ExcelImportUtil.importExcel(file.getInputStream(), CTopOauthToken.class, params);
              cTopOauthTokenService.saveBatch(listCTopOauthTokens);
              return Result.ok("文件导入成功！数据行数:" + listCTopOauthTokens.size());
          } catch (Exception e) {
              log.error(e.getMessage(),e);
              return Result.error("文件导入失败:"+e.getMessage());
          } finally {
              try {
                  file.getInputStream().close();
              } catch (IOException e) {
                  e.printStackTrace();
              }
          }
      }
      return Result.ok("文件导入失败！");
  }

}
