package org.jeecg.modules.ctop.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import org.apache.shiro.SecurityUtils;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.common.system.vo.LoginUser;
import org.jeecg.modules.ctop.entity.*;
import org.jeecg.modules.ctop.mapper.ByteDanceAdvertiserMapper;
import org.jeecg.modules.ctop.mapper.ByteDanceBudgetTemplateMapper;
import org.jeecg.modules.ctop.service.IByteDanceAdvertiserBudgetService;
import org.jeecg.modules.ctop.service.IByteDanceAdvertiserService;
import org.jeecg.modules.ctop.service.IByteDanceBudgetTemplateService;
import org.jeecg.modules.ctop.service.ICTopOauthTokenService;
import org.omg.CORBA.OBJ_ADAPTER;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;

import javax.servlet.http.HttpServletRequest;
import java.util.HashMap;
import java.util.Map;

/**
 * @Description: 今日头条预算模板信息
 * @Author: jeecg-boot
 * @Date: 2019-07-22
 * @Version: V1.0
 */
@Service
public class ByteDanceBudgetTemplateServiceImpl extends ServiceImpl<ByteDanceBudgetTemplateMapper, ByteDanceBudgetTemplate> implements IByteDanceBudgetTemplateService {

    @Override
    public Map<String, Object> insertTemplate(ByteDanceBudgetTemplate template) {
        Map<String, Object> resultMap = new HashMap<>();
        LoginUser user = (LoginUser) SecurityUtils.getSubject().getPrincipal();
        CTopOauthToken token = tokenService.getOAuthTokenByAccountId(user.getId());
        template.setAdvertiserId(token.getAdvertiserId());
        template.setToutiaoId(token.getAccountId());
        template.setStatus(1);
        template.setStartDate(template.getStartDate().substring(0, 10));
        template.setEndDate(template.getEndDate().substring(0, 10));
        budgetTemplateMapper.insert(template);
        resultMap.put("success", true);
        resultMap.put("message", "模板保存成功");
        return resultMap;
    }

    @Override
    public Result<IPage<ByteDanceBudgetTemplate>> getList(ByteDanceBudgetTemplate template, Integer pageNo, Integer pageSize, HttpServletRequest req) {
        LoginUser user = (LoginUser) SecurityUtils.getSubject().getPrincipal();
        CTopOauthToken token = tokenService.getOAuthTokenByAccountId(user.getId());
        Result<IPage<ByteDanceBudgetTemplate>> result = new Result<>();
        QueryWrapper<ByteDanceBudgetTemplate> queryWrapper = QueryGenerator.initQueryWrapper(template, req.getParameterMap());
        queryWrapper.eq("advertiser_id", token.getAdvertiserId()).orderByDesc("create_time");
        Page<ByteDanceBudgetTemplate> page = new Page<>(pageNo, pageSize);
        IPage<ByteDanceBudgetTemplate> pageList = this.page(page, queryWrapper);
        result.setSuccess(true);
        result.setResult(pageList);
        return result;
    }

    @Autowired
    private ICTopOauthTokenService tokenService;
    @Autowired
    private ByteDanceBudgetTemplateMapper budgetTemplateMapper;
}
