package org.jeecg.modules.ctop.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import org.apache.shiro.SecurityUtils;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.common.system.vo.LoginUser;
import org.jeecg.modules.ctop.entity.ByteDanceCampaign;
import org.jeecg.modules.ctop.entity.ByteDanceCampaignTemplate;
import org.jeecg.modules.ctop.entity.CTopOauthToken;
import org.jeecg.modules.ctop.mapper.ByteDanceCampaignTemplateMapper;
import org.jeecg.modules.ctop.service.IByteDanceCampaignTemplateService;
import org.jeecg.modules.ctop.service.ICTopOauthTokenService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;

import javax.servlet.http.HttpServletRequest;
import java.util.HashMap;
import java.util.Map;

/**
 * @Description: 今日头条广告组模板信息
 * @Author: jeecg-boot
 * @Date: 2019-07-22
 * @Version: V1.0
 */
@Service
public class ByteDanceCampaignTemplateServiceImpl extends ServiceImpl<ByteDanceCampaignTemplateMapper, ByteDanceCampaignTemplate> implements IByteDanceCampaignTemplateService {

    @Override
    public Map<String, Object> insertTemplate(ByteDanceCampaignTemplate template) {
        Map<String, Object> resultMap = new HashMap<>();
        LoginUser user = (LoginUser) SecurityUtils.getSubject().getPrincipal();
        CTopOauthToken token = tokenService.getOAuthTokenByAccountId(user.getId());
        template.setAccountId(token.getAccountId());
        template.setAdvertiserId(token.getAdvertiserId());
        campaignTemplateMapper.insert(template);
        resultMap.put("success", true);
        resultMap.put("message", "模板保存成功");
        return resultMap;
    }

    @Override
    public Result<IPage<ByteDanceCampaignTemplate>> getList(ByteDanceCampaignTemplate template, Integer pageNo, Integer pageSize, HttpServletRequest req) {
        LoginUser user = (LoginUser) SecurityUtils.getSubject().getPrincipal();
        CTopOauthToken token = tokenService.getOAuthTokenByAccountId(user.getId());
        Result<IPage<ByteDanceCampaignTemplate>> result = new Result<>();
        QueryWrapper<ByteDanceCampaignTemplate> queryWrapper = QueryGenerator.initQueryWrapper(template, req.getParameterMap());
        queryWrapper.eq("advertiser_id", token.getAdvertiserId()).orderByDesc("create_time");
        Page<ByteDanceCampaignTemplate> page = new Page<>(pageNo, pageSize);
        IPage<ByteDanceCampaignTemplate> pageList = this.page(page, queryWrapper);
        result.setSuccess(true);
        result.setResult(pageList);
        return result;
    }

    @Autowired
    private ByteDanceCampaignTemplateMapper campaignTemplateMapper;
    @Autowired
    private ICTopOauthTokenService tokenService;
}
