package org.jeecg.modules.ctop.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import org.apache.shiro.SecurityUtils;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.common.system.vo.LoginUser;
import org.jeecg.modules.ctop.entity.ByteDanceCampaignTemplate;
import org.jeecg.modules.ctop.entity.BytedanceDeliveryTargetTemplate;
import org.jeecg.modules.ctop.entity.CTopOauthToken;
import org.jeecg.modules.ctop.mapper.BytedanceDeliveryTargetTemplateMapper;
import org.jeecg.modules.ctop.service.IBytedanceDeliveryTargetTemplateService;
import org.jeecg.modules.ctop.service.ICTopOauthTokenService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;

import javax.servlet.http.HttpServletRequest;
import java.util.HashMap;
import java.util.Map;

/**
 * @Description: 投放目标模板信息
 * @Author: jeecg-boot
 * @Date: 2019-07-23
 * @Version: V1.0
 */
@Service
public class BytedanceDeliveryTargetTemplateServiceImpl extends ServiceImpl<BytedanceDeliveryTargetTemplateMapper, BytedanceDeliveryTargetTemplate> implements IBytedanceDeliveryTargetTemplateService {

    @Override
    public Map<String, Object> insertTemplate(BytedanceDeliveryTargetTemplate template) {
        Map<String, Object> resultMap = new HashMap<>();
        LoginUser user = (LoginUser) SecurityUtils.getSubject().getPrincipal();
        CTopOauthToken token = tokenService.getOAuthTokenByAccountId(user.getId());
        template.setAccountId(token.getAccountId());
        template.setAdvertiserId(token.getAdvertiserId());
        template.setStatus(1);
        deliveryTargetTemplateMapper.insert(template);
        resultMap.put("success", true);
        resultMap.put("message", "投放目标模板保存成功");
        return resultMap;
    }

    @Override
    public Result<IPage<BytedanceDeliveryTargetTemplate>> getList(BytedanceDeliveryTargetTemplate template, Integer pageNo, Integer pageSize, HttpServletRequest req) {
        LoginUser user = (LoginUser) SecurityUtils.getSubject().getPrincipal();
        CTopOauthToken token = tokenService.getOAuthTokenByAccountId(user.getId());
        Result<IPage<BytedanceDeliveryTargetTemplate>> result = new Result<>();
        QueryWrapper<BytedanceDeliveryTargetTemplate> queryWrapper = QueryGenerator.initQueryWrapper(template, req.getParameterMap());
        queryWrapper.eq("advertiser_id", token.getAdvertiserId()).orderByDesc("create_time");
        Page<BytedanceDeliveryTargetTemplate> page = new Page<>(pageNo, pageSize);
        IPage<BytedanceDeliveryTargetTemplate> pageList = this.page(page, queryWrapper);
        result.setSuccess(true);
        result.setResult(pageList);
        return result;
    }

    @Autowired
    private ICTopOauthTokenService tokenService;
    @Autowired
    private BytedanceDeliveryTargetTemplateMapper deliveryTargetTemplateMapper;
}
