package org.jeecg.modules.kuaishou.controller;

import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.common.aspect.annotation.AutoLog;
import org.jeecg.common.util.oConvertUtils;
import org.jeecg.modules.kuaishou.entity.KuaiShouGroupCreate;
import org.jeecg.modules.kuaishou.service.IKuaiShouGroupCreateService;
import java.util.Date;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import lombok.extern.slf4j.Slf4j;

import org.jeecgframework.poi.excel.ExcelImportUtil;
import org.jeecgframework.poi.excel.def.NormalExcelConstants;
import org.jeecgframework.poi.excel.entity.ExportParams;
import org.jeecgframework.poi.excel.entity.ImportParams;
import org.jeecgframework.poi.excel.view.JeecgEntityExcelView;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import org.springframework.web.servlet.ModelAndView;
import com.alibaba.fastjson.JSON;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

 /**
  * @Description: 快手-创建广告组信息
 * @Author: jeecg-boot
  * @Date: 2019-07-24
 * @Version: V1.0
 */
@Slf4j
@Api(tags = "快手-创建广告组信息")
@RestController
@RequestMapping("/kuaishou/kuaiShouGroupCreate")
public class KuaiShouGroupCreateController {
	@Autowired
    private IKuaiShouGroupCreateService kuaiShouGroupCreateService;
	
	/**
	  * 分页列表查询
     * @param kuaiShouGroupCreate
	 * @param pageNo
	 * @param pageSize
	 * @param req
	 * @return
	 */
    @AutoLog(value = "快手-创建广告组信息-分页列表查询")
    @ApiOperation(value = "快手-创建广告组信息-分页列表查询", notes = "快手-创建广告组信息-分页列表查询")
	@GetMapping(value = "/list")
    public Result<IPage<KuaiShouGroupCreate>> queryPageList(KuaiShouGroupCreate kuaiShouGroupCreate,
                                                            @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                            @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                                            HttpServletRequest req) {
        Result<IPage<KuaiShouGroupCreate>> result = new Result<IPage<KuaiShouGroupCreate>>();
        QueryWrapper<KuaiShouGroupCreate> queryWrapper = QueryGenerator.initQueryWrapper(kuaiShouGroupCreate, req.getParameterMap());
        Page<KuaiShouGroupCreate> page = new Page<KuaiShouGroupCreate>(pageNo, pageSize);
        IPage<KuaiShouGroupCreate> pageList = kuaiShouGroupCreateService.page(page, queryWrapper);
		result.setSuccess(true);
		result.setResult(pageList);
		return result;
	}
	
	/**
	  *   添加
     * @param kuaiShouGroupCreate
	 * @return
	 */
    @AutoLog(value = "快手-创建广告组信息-添加")
    @ApiOperation(value = "快手-创建广告组信息-添加", notes = "快手-创建广告组信息-添加")
	@PostMapping(value = "/add")
    public Result<KuaiShouGroupCreate> add(@RequestBody KuaiShouGroupCreate kuaiShouGroupCreate) {
        Result<KuaiShouGroupCreate> result = new Result<KuaiShouGroupCreate>();
		try {
            kuaiShouGroupCreateService.save(kuaiShouGroupCreate);
			result.success("添加成功！");
		} catch (Exception e) {
			log.error(e.getMessage(),e);
			result.error500("操作失败");
		}
		return result;
	}
	
	/**
	  *  编辑
     * @param kuaiShouGroupCreate
	 * @return
	 */
    @AutoLog(value = "快手-创建广告组信息-编辑")
    @ApiOperation(value = "快手-创建广告组信息-编辑", notes = "快手-创建广告组信息-编辑")
	@PutMapping(value = "/edit")
    public Result<KuaiShouGroupCreate> edit(@RequestBody KuaiShouGroupCreate kuaiShouGroupCreate) {
        Result<KuaiShouGroupCreate> result = new Result<KuaiShouGroupCreate>();
        KuaiShouGroupCreate kuaiShouGroupCreateEntity = kuaiShouGroupCreateService.getById(kuaiShouGroupCreate.getId());
        if (kuaiShouGroupCreateEntity == null) {
			result.error500("未找到对应实体");
		}else {
            boolean ok = kuaiShouGroupCreateService.updateById(kuaiShouGroupCreate);
			//TODO 返回false说明什么？
			if(ok) {
				result.success("修改成功!");
			}
		}
		
		return result;
	}
	
	/**
	  *   通过id删除
	 * @param id
	 * @return
	 */
    @AutoLog(value = "快手-创建广告组信息-通过id删除")
    @ApiOperation(value = "快手-创建广告组信息-通过id删除", notes = "快手-创建广告组信息-通过id删除")
	@DeleteMapping(value = "/delete")
	public Result<?> delete(@RequestParam(name="id",required=true) String id) {
		try {
            kuaiShouGroupCreateService.removeById(id);
		} catch (Exception e) {
			log.error("删除失败",e.getMessage());
			return Result.error("删除失败!");
		}
		return Result.ok("删除成功!");
	}
	
	/**
	  *  批量删除
	 * @param ids
	 * @return
	 */
    @AutoLog(value = "快手-创建广告组信息-批量删除")
    @ApiOperation(value = "快手-创建广告组信息-批量删除", notes = "快手-创建广告组信息-批量删除")
	@DeleteMapping(value = "/deleteBatch")
    public Result<KuaiShouGroupCreate> deleteBatch(@RequestParam(name = "ids", required = true) String ids) {
        Result<KuaiShouGroupCreate> result = new Result<KuaiShouGroupCreate>();
		if(ids==null || "".equals(ids.trim())) {
			result.error500("参数不识别！");
		}else {
            this.kuaiShouGroupCreateService.removeByIds(Arrays.asList(ids.split(",")));
			result.success("删除成功!");
		}
		return result;
	}
	
	/**
	  * 通过id查询
	 * @param id
	 * @return
	 */
    @AutoLog(value = "快手-创建广告组信息-通过id查询")
    @ApiOperation(value = "快手-创建广告组信息-通过id查询", notes = "快手-创建广告组信息-通过id查询")
	@GetMapping(value = "/queryById")
    public Result<KuaiShouGroupCreate> queryById(@RequestParam(name = "id", required = true) String id) {
        Result<KuaiShouGroupCreate> result = new Result<KuaiShouGroupCreate>();
        KuaiShouGroupCreate kuaiShouGroupCreate = kuaiShouGroupCreateService.getById(id);
        if (kuaiShouGroupCreate == null) {
			result.error500("未找到对应实体");
		}else {
            result.setResult(kuaiShouGroupCreate);
			result.setSuccess(true);
		}
		return result;
	}

  /**
      * 导出excel
   *
   * @param request
   * @param response
   */
  @RequestMapping(value = "/exportXls")
  public ModelAndView exportXls(HttpServletRequest request, HttpServletResponse response) {
      // Step.1 组装查询条件
      QueryWrapper<KuaiShouGroupCreate> queryWrapper = null;
      try {
          String paramsStr = request.getParameter("paramsStr");
          if (oConvertUtils.isNotEmpty(paramsStr)) {
              String deString = URLDecoder.decode(paramsStr, "UTF-8");
              KuaiShouGroupCreate kuaiShouGroupCreate = JSON.parseObject(deString, KuaiShouGroupCreate.class);
              queryWrapper = QueryGenerator.initQueryWrapper(kuaiShouGroupCreate, request.getParameterMap());
          }
      } catch (UnsupportedEncodingException e) {
          e.printStackTrace();
      }

      //Step.2 AutoPoi 导出Excel
      ModelAndView mv = new ModelAndView(new JeecgEntityExcelView());
      List<KuaiShouGroupCreate> pageList = kuaiShouGroupCreateService.list(queryWrapper);
      //导出文件名称
      mv.addObject(NormalExcelConstants.FILE_NAME, "快手-创建广告组信息列表");
      mv.addObject(NormalExcelConstants.CLASS, KuaiShouGroupCreate.class);
      mv.addObject(NormalExcelConstants.PARAMS, new ExportParams("快手-创建广告组信息列表数据", "导出人:Jeecg", "导出信息"));
      mv.addObject(NormalExcelConstants.DATA_LIST, pageList);
      return mv;
  }

  /**
      * 通过excel导入数据
   *
   * @param request
   * @param response
   * @return
   */
  @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
  public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
      MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
      Map<String, MultipartFile> fileMap = multipartRequest.getFileMap();
      for (Map.Entry<String, MultipartFile> entity : fileMap.entrySet()) {
          MultipartFile file = entity.getValue();// 获取上传文件对象
          ImportParams params = new ImportParams();
          params.setTitleRows(2);
          params.setHeadRows(1);
          params.setNeedSave(true);
          try {
              List<KuaiShouGroupCreate> listKuaiShouGroupCreates = ExcelImportUtil.importExcel(file.getInputStream(), KuaiShouGroupCreate.class, params);
              kuaiShouGroupCreateService.saveBatch(listKuaiShouGroupCreates);
              return Result.ok("文件导入成功！数据行数:" + listKuaiShouGroupCreates.size());
          } catch (Exception e) {
              log.error(e.getMessage(),e);
              return Result.error("文件导入失败:"+e.getMessage());
          } finally {
              try {
                  file.getInputStream().close();
              } catch (IOException e) {
                  e.printStackTrace();
              }
          }
      }
      return Result.ok("文件导入失败！");
  }

}
