package org.jeecg.modules.kuaishou.service;


import cn.com.ctop.kuaishou.modules.material.entity.KuaishouResult;
import cn.com.ctop.kuaishou.modules.material.entity.KuaishouResultToken;
import cn.com.ctop.kuaishou.modules.report.entity.*;
import com.alibaba.fastjson.JSONObject;
import org.springframework.http.HttpRequest;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import java.util.Map;

public interface IKuaishouInterfaceService {
    KuaishouResult<KuaishouReportHourlyAccount> getAccountHourlyReport(String accessToken, Long advertiserId, String startDate, String endDate, int pageSize, int pageNum);

    KuaishouResult<KuaishouReportDailyAccount> getAccountDailyReport(String accessToken, Long advertiserId, String startDate, String endDate, int pageSize, int pageNum);

    KuaishouResult<KuaishouReportDailyCampaign> getCampaignDailyReport(String accessToken, Long advertiserId, String startDate, String endDate, int pageSize, int pageNum);

    KuaishouResult<KuaishouReportHourlyCampaign> getCampaignHourlyReport(String accessToken, Long advertiserId, String startDate, String endDate, int pageSize, int pageNum);

    KuaishouResult<KuaishouReportDailyGroup> getGroupDailyReport(String accessToken, Long advertiserId, String startDate, String endDate, int pageSize, int pageNum);

    KuaishouResult<KuaishouReportHourlyGroup> getGroupHourlyReport(String accessToken, Long advertiserId, String startDate, String endDate, int pageSize, int pageNum);

    KuaishouResult<KuaishouReportDailyCreative> getCreativeDailyReport(String accessToken, Long advertiserId, String startDate, String endDate, int pageSize, int pageNum);

    KuaishouResult<KuaishouReportHourlyCreative> getCreativeHourlyReport(String accessToken, Long advertiserId, String startDate, String endDate, int pageSize, int pageNum);


    /**
     * 创建广告计划
     *
     * @param accessToken
     * @param advertiser_id
     * @param requestJson
     * @return
     */
    Map<String, Object> campaignCreate(String accessToken, Long advertiser_id, JSONObject requestJson);


    /**
     * 创建广告组
     *
     * @param accessToken
     * @param advertiserId
     * @param requestJson
     * @return
     */
    Map<String, Object> adUnitCreate(String accessToken, Long advertiserId, JSONObject requestJson);


    /**
     * 获取广告计划信息
     *
     * @param accessToken
     * @param advertiserId
     */
    void getCampaignList(String accessToken, Long advertiserId, Integer page);

    /**
     * 获取广告组信息
     *
     * @param accessToken
     * @param advertiserId
     * @param page
     */
    void getGroupList(String accessToken, Long advertiserId, Integer page);

    /**
     * 获取创意信息
     *
     * @param accessToken
     * @param advertiserId
     */
    void getCreativeList(String accessToken, Long advertiserId, Integer page);

    /**
     * 获取token
     *
     * @param authCode
     * @return
     */
    KuaishouResultToken getAccessToken(String authCode);

    /**
     * 刷新token
     *
     * @param refreshToken
     * @return
     */
    KuaishouResultToken getRefreshToken(String refreshToken);


    /**
     * 获取可选的深度转化类型
     *
     * @param advertiserId
     */
    void getDeepConversionInfos(Long advertiserId, String accessToken);


    /**
     * 创建广告创意
     *
     * @param accessToken
     * @param advertiserId
     * @param requestJson
     */
    Map<String, Object> creativeCreate(String accessToken, Long advertiserId, JSONObject requestJson);

    /**
     * 文件上传
     *
     * @param advertiserId
     * @param accessToken
     * @param multipartFile
     */
    void videoUpload(Long advertiserId, String accessToken, MultipartFile multipartFile, HttpServletRequest request);

    /**
     * 获取视频
     *
     * @param advertiserId
     * @param accessToken
     * @param photoId
     */
    void videoGet(Long advertiserId, String accessToken, String photoId);

    /**
     * 获取图片信息
     *
     * @param advertiserId
     * @param accessToken
     * @param image_token
     */
    void imageGet(Long advertiserId, String accessToken, String image_token);

    /**
     * 获取 地域列表
     *
     * @param accessToken
     */
    void areaList(String accessToken);


    /**
     * 图片上传
     *
     * @param advertiserId
     * @param accessToken
     * @param multipartFile
     * @param request
     */
    void imageUpload(Long advertiserId, String accessToken, MultipartFile multipartFile, HttpServletRequest request);


    /**
     * 创建应用
     *
     * @param advertiserId
     * @param accessToken
     * @param multipartFile
     * @param request
     */
    void appCreate(Long advertiserId, String accessToken, MultipartFile multipartFile, HttpServletRequest request, JSONObject requestJson);


    /**
     * 获取应用列表
     *
     * @param advertiserId
     * @param accessToken
     */
    void getAppList(Long advertiserId, String accessToken);

    /**
     * 文件上传 发送请求
     *
     * @param url
     * @param paramMap
     * @param headerMap
     * @param filePath
     */
    String exceptInfoForRestTemplate(String url, Map<String, Object> paramMap, Map<String, String> headerMap, String filePath);


    /**
     * 修改应用
     *
     * @param advertiserId
     * @param accessToken
     * @param requestJson
     * @param multipartFile
     * @param request
     */
    void appUpdate(Long advertiserId, String accessToken, JSONObject requestJson, MultipartFile multipartFile, HttpServletRequest request);
}
