package cn.com.ctop.kuaishou.modules.graphql.service.impl;

import cn.com.ctop.common.utils.HttpUtils;
import cn.com.ctop.kuaishou.modules.graphql.entity.*;
import cn.com.ctop.kuaishou.modules.graphql.service.IKuaishouUserService;
import cn.com.ctop.kuaishou.modules.graphql.service.IKuaishouVideoService;
import cn.com.ctop.kuaishou.modules.graphql.service.IKuaishouWebInterfaceService;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.DeserializationFeature;
import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.google.gson.Gson;
import org.apache.http.client.CookieStore;
import org.apache.http.impl.client.BasicCookieStore;
import org.apache.http.impl.cookie.BasicClientCookie;
import org.openqa.selenium.By;
import org.openqa.selenium.Cookie;
import org.openqa.selenium.WebDriver;
import org.openqa.selenium.chrome.ChromeDriver;
import org.openqa.selenium.chrome.ChromeOptions;
import org.openqa.selenium.support.ui.ExpectedConditions;
import org.openqa.selenium.support.ui.WebDriverWait;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.regex.Matcher;
import java.util.regex.Pattern;


@Service
public class KuaishouWebInterfaceServiceImpl implements IKuaishouWebInterfaceService {
    @Autowired
    private IKuaishouUserService kuaishouUserService;
    @Autowired
    private IKuaishouVideoService kuaishouVideoService;
    public static String GRAPHQL_URL = "https://live.kuaishou.com/graphql";

    public void getSearchUserList(String keyword, int page, String ttf) {
        String jsonStr = "{\"operationName\":\"SearchDetailQuery\",\"variables\":{\"key\":\""+keyword+"\",\"type\":\"author\",\"page\":"+String.valueOf(page)+",\"lssid\":null,\"ussid\":\"MV8wXzE1NjA4NTgzMjMzODdf5ri45oiPXzk3MTE\"},\"query\":\"query SearchDetailQuery($key: String, $type: String, $page: Int, $lssid: String, $ussid: String) {  searchDetail(key: $key, type: $type, page: $page, lssid: $lssid, ussid: $ussid) {    ... on SearchCategoryList {      type      list {        id        categoryId        title        src        roomNumber        __typename      }      __typename    }    ... on SearchUserList {      type      ussid      list {        id        name        living        profile        sex        description        countsInfo {          fan          follow          photo          __typename        }        __typename      }      __typename    }    ... on SearchLivestreamList {      type      lssid      list {        user {          id          profile          name          __typename        }        watchingCount        src        title        gameId        gameName        categoryId        liveStreamId        playUrls {          quality          url          __typename        }        quality        gameInfo {          category          name          pubgSurvival          type          kingHero          __typename        }        redPack        liveGuess        expTag        __typename      }      __typename    }    __typename  }}\"}";
        try {
            if(page < 1){
                page = 1;
            }
            String result = HttpUtils.callingGraph(GRAPHQL_URL,jsonStr);
            Map<String,String> map = HttpUtils.fontsMap.get(ttf);
            for(String key : map.keySet()){
                result = result.replace(key,map.get(key));
            }
            System.out.println(result);
            ObjectMapper mapper = new ObjectMapper();
            mapper.configure(DeserializationFeature.FAIL_ON_UNKNOWN_PROPERTIES, false);
            KuaishouWebResult<KuaishouSearchResult> data = mapper.readValue(result,new TypeReference<KuaishouWebResult<KuaishouSearchResult>>(){});
            List<KuaishouAuthors> list = data.getData().getSearchDetail().getList();
            if(list != null && list.size() > 0){
                System.out.println("list size="+list.size());
                Thread.sleep((int)Math.random()*5000+5000);
                getSearchUserList(keyword,++page,ttf);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
    public void getVedioList(String principalId,String pcursor,String ttf,int times){
        String jsonStr = "{" +
                "\"operationName\":\"publicFeedsQuery\"," +
                "\"variables\":{\"principalId\":\""+principalId+"\",\"pcursor\":\""+pcursor+"\",\"count\":24}," +
                "\"query\":\"query publicFeedsQuery($principalId: String, $pcursor: String, $count: Int) " +
                "{" +
                "publicFeeds(principalId: $principalId, pcursor: $pcursor, count: $count) " +
                "{    " +
                "pcursor    " +
                "live " +
                "{      " +
                "user " +
                "{        " +
                "id        " +
                "kwaiId        " +
                "eid        " +
                "profile        " +
                "name        " +
                "living        " +
                "}      " +
                "watchingCount      " +
                "src      " +
                "title      " +
                "gameId      " +
                "gameName      " +
                "categoryId      " +
                "liveStreamId      " +
                "playUrls " +
                "{        " +
                "quality        " +
                "url        " +
                "}      " +
                "followed      " +
                "type      " +
                "living      " +
                "redPack      " +
                "liveGuess      " +
                "anchorPointed      " +
                "latestViewed      " +
                "expTag      " +
                "}    " +
                "list " +
                "{      " +
                "photoId      " +
                "caption      " +
                "thumbnailUrl      " +
                "poster      " +
                "viewCount      " +
                "likeCount      " +
                "commentCount      " +
                "timestamp      " +
                "workType      " +
                "type      " +
                "useVideoPlayer      " +
                "imgUrls      " +
                "imgSizes      " +
                "magicFace      " +
                "musicName      " +
                "location      " +
                "liked      " +
                "onlyFollowerCanComment      " +
                "relativeHeight      " +
                "width      " +
                "height      " +
                "user " +
                "{        " +
                "id        " +
                "eid        " +
                "name        " +
                "profile        " +
                "}      " +
                "expTag      " +
                "}    " +
                "}" +
                "}\"" +
                "}";
        try {
            if (times > 5){
                return;
            }
            String result = HttpUtils.callingGraph(GRAPHQL_URL,jsonStr);
            Map<String,String> map = HttpUtils.fontsMap.get(ttf);
            for(String key : map.keySet()){
                result = result.replace(key,map.get(key));
            }
            ObjectMapper mapper = new ObjectMapper();
            mapper.configure(DeserializationFeature.FAIL_ON_UNKNOWN_PROPERTIES, false);

            KuaishouWebResult<KuaishouWebData> data = mapper.readValue(result,new TypeReference<KuaishouWebResult<KuaishouWebData>>(){});
            KuaishouPublicFeeds feeds = data.getData().getPublicFeeds();
            if(feeds.getList() != null && feeds.getList().size() > 0){
                List<KuaishouVideo> videoList = feeds.getList();
                for(KuaishouVideo video : videoList){
                    video.setUserId(video.getUser().getId());
                    video.setTimes(new Date(video.getTimestamp()));
                    video.setTtfName(ttf);
                    kuaishouUserService.saveOrUpdate(video.getUser());
                    kuaishouVideoService.saveOrUpdate(video);
                }
                System.out.println("list size="+feeds.getList().size());
            }
            if (feeds.getPcursor() == null || !feeds.getPcursor().equals("no_more")){
                Thread.sleep((int)Math.random()*5000+10000);
                getVedioList(principalId,feeds.getPcursor(),ttf,++times);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public void getSecurity(String url){
        String detectionPoint = "{\"a\":false,\"b\":true,\"c\":true,\"d\":true,\"e\":false,\"f\":false,\"g\":false,\"h\":true,\"i\":true,\"j\":false,\"k\":false,\"l\":true,\"m\":true,\"n\":true,\"o\":false,\"p\":false,\"q\":true,\"r\":false,\"s\":false,\"t\":false,\"u\":false,\"v\":false,\"w\":false,\"x\":false,\"y\":false,\"z\":true,\"aa\":false,\"bb\":false,\"cc\":false,\"dd\":false,\"ee\":false,\"ff\":false,\"gg\":false,\"hh\":false,\"ii\":false,\"jj\":false,\"kk\":false,\"ll\":false,\"mm\":false,\"nn\":false,\"oo\":false,\"pp\":1}";
        String collectInfo="{\"a\":\"live.kuaishou.com\",\"b\":\""+url+"\",\"c\":\"null的主页-快手直播\",\"d\":\"\",\"e\":\""+HttpUtils.USER_AGENT+"\",\"f\":1920,\"g\":1080,\"h\":24,\"i\":\"zh-CN\"}";
        try {
            String securityUrl = "https://live.kuaishou.com/rest/wd/live/web/security?detectionPoint=" + URLEncoder.encode(detectionPoint, "UTF-8") + "&collectInfo=" + URLEncoder.encode(collectInfo, "UTF-8");
            String html2 = HttpUtils.httpGetRequest(securityUrl);
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }
    }

    public void adkuaishouWebLogin(String phone, String password) {
        System.getProperties().setProperty("webdriver.chrome.driver", "D:/chromedriver.exe");
        try {
            ChromeOptions chromeOptions = new ChromeOptions();
            WebDriver webDriver = new ChromeDriver(chromeOptions);
            chromeOptions.addArguments("--headless");
            chromeOptions.addArguments("--no-sandbox");
            chromeOptions.addArguments("--disable-gpu");
            chromeOptions.addArguments("--window-size=1290,1080");
            chromeOptions.addArguments("--user-agent=" + HttpUtils.USER_AGENT);
            chromeOptions.setAcceptInsecureCerts(true);
            //全局隐式等待
            webDriver.manage().timeouts().implicitlyWait(10, TimeUnit.SECONDS);
            //设定网址
            webDriver.get("https://b.e.kuaishou.com");
            //显示等待控制对象
            WebDriverWait webDriverWait = new WebDriverWait(webDriver, 10);
            //等待输入框可用后输入账号密码
            for (Cookie cookie : webDriver.manage().getCookies()) {
                System.out.println(cookie.getName());
                System.out.println(cookie.getValue());
            }
//        String html = webDriver.getPageSource();
//        System.out.println(html);
            webDriverWait.until(ExpectedConditions.elementToBeClickable(By.cssSelector(".phone input"))).sendKeys(phone);
            webDriverWait.until(ExpectedConditions.elementToBeClickable(By.cssSelector(".password input"))).sendKeys(password);
            //点击登录
            webDriver.findElement(By.className("foot")).click();
            //等待2秒用于页面加载，保证Cookie响应全部获取。
            Thread.sleep(2000);
            webDriver.get("https://b.e.kuaishou.com/#/home/profile");
            //获取Cookie并打印
            Set<Cookie> cookies = webDriver.manage().getCookies();
            Iterator iterator = cookies.iterator();
            CookieStore cookieStore = new BasicCookieStore();
            String infoUrl = "https://b.e.kuaishou.com/rest/business/commentInfo/photos";
            Map<String, String> parameterMap = new HashMap<String, String>();
            parameterMap.put("pcursorMap", new Gson().toJson(new HashMap<>()));
            parameterMap.put("count", "30");
            while (iterator.hasNext()) {

                String cookieStr = iterator.next().toString();
                System.out.println(cookieStr);
                String[] cookieArray = cookieStr.split(";");
                Map<String, String> cookieMap = new HashMap<String, String>();
                for (String cookie : cookieArray) {
                    String[] kv = cookie.split("=");
                    if (kv.length > 1) {
                        cookieMap.put(kv[0].trim(), kv[1].trim());
                    }
                }
                BasicClientCookie clientCookie = new BasicClientCookie(cookieArray[0].split("=")[0],
                        cookieArray[0].split("=")[1]);
                clientCookie.setDomain("b.e.kuaishou.com");
//                clientCookie.setDomain(cookieMap.get("domain"));
                clientCookie.setPath(cookieMap.get("path"));
                HttpUtils.cookieStore.addCookie(clientCookie);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public void getVideoList(String uid){
//        System.getProperties().setProperty("webdriver.chrome.driver", "D:/chromedriver.exe");
        System.getProperties().setProperty("webdriver.chrome.driver", "/usr/bin/chromedriver");
        try {
            ChromeOptions chromeOptions = new ChromeOptions();
            WebDriver webDriver = new ChromeDriver(chromeOptions);
            chromeOptions.addArguments("--headless");
            chromeOptions.addArguments("--no-sandbox");
            chromeOptions.addArguments("--disable-gpu");
            chromeOptions.addArguments("--window-size=1290,1080");
            chromeOptions.addArguments("--user-agent="+HttpUtils.USER_AGENT);
            chromeOptions.setAcceptInsecureCerts(true);
            webDriver.get("https://live.kuaishou.com/profile/"+uid);
            Thread.sleep(2000);
            webDriver.navigate().refresh();
            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
            for (Cookie cookie : webDriver.manage().getCookies()){
                BasicClientCookie ck = new BasicClientCookie(cookie.getName(),cookie.getValue());
                ck.setDomain("live.kuaishou.com");
//                ck.setDomain(cookie.getDomain());
                ck.setExpiryDate(cookie.getExpiry());
                ck.setPath(cookie.getPath());
                HttpUtils.cookieStore.addCookie(ck);
           }
            String html = webDriver.getPageSource();
            Pattern p=Pattern.compile("fontscn\\w+\\.ttf");
            Matcher m=p.matcher(html);

            String ttf = "";
            while(m.find()) {
                ttf = m.group();
                System.out.println("ttf:"+ttf);
            }
            Map<String,String> map = HttpUtils.fontsMap.get(ttf);
            if(map == null){
                HttpUtils.kuaishouTtf(ttf);
            }
            getVedioList(uid,"",ttf,1);
            webDriver.close();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private void setCookies(){
        HttpUtils.cookieStore.clear();
        KuaishouWebInterfaceServiceImpl i= new KuaishouWebInterfaceServiceImpl();
        BasicClientCookie cookie = new BasicClientCookie("clientid", "3");
        cookie.setDomain("live.kuaishou.com");   //设置范围
        cookie.setPath("/");
        cookie.setExpiryDate(new Date(new Date().getTime()+1000*60*60*24*365));
        HttpUtils.cookieStore.addCookie(cookie);

        BasicClientCookie cookie1 = new BasicClientCookie("did", "web_1615d3ad757d4fb8558ed4d7dbecf5fc");
        cookie1.setDomain("live.kuaishou.com");   //设置范围web_a5735c4f9f02ad9ee48677499927341c
        cookie1.setPath("/");
        cookie1.setExpiryDate(new Date(new Date().getTime()+1000*60*60*24*365));
        HttpUtils.cookieStore.addCookie(cookie1);

        BasicClientCookie cookie2 = new BasicClientCookie("client_key", "65890b29");
        cookie2.setDomain("live.kuaishou.com");   //设置范围
        cookie2.setPath("/");
        cookie2.setExpiryDate(new Date(new Date().getTime()+1000*60*60*24*365));
        HttpUtils.cookieStore.addCookie(cookie2);

        BasicClientCookie cookie3 = new BasicClientCookie("kuaishou.live.bfb1s", "3e261140b0cf7444a0ba411c6f227d88");
        cookie3.setDomain("live.kuaishou.com");   //设置范围
        cookie3.setPath("/");
        cookie3.setExpiryDate(new Date(new Date().getTime()+1000*60*60*24*365));
        HttpUtils.cookieStore.addCookie(cookie3);
    }

    public String checkKuaishouUser(String videoUrl){
        String html = HttpUtils.httpGetRequest(videoUrl);
        HttpUtils.cookieStore.clear();
        Pattern p=Pattern.compile("/profile/(\\w+)\"");
        Matcher m=p.matcher(html);
        String uid = "";

        while(m.find()) {
            uid = m.group().replace("/profile/","").replace("\"","");
            if (uid != null && !uid.equals("")&& !uid.equals("null")){
                return uid;
            }
        }
        return null;
    }

    public void deleteCommentById(Long photoId, Long commentId, Map<String, String> headers) {
        try {
            Map<String, Object> params = new HashMap<>();
            params.put("photoId", photoId);
            params.put("commentId", commentId);
            params.put("principalUserId", 0);
            String str = HttpUtils.httpPostRequest("https://b.e.kuaishou.com/rest/kd/ad/comment/deleteV2", params, headers);
            ObjectMapper mapper = new ObjectMapper();
            mapper.configure(DeserializationFeature.FAIL_ON_UNKNOWN_PROPERTIES, false);
            JsonNode node = mapper.readTree(str);
            Integer result = node.get("result").asInt();
            if (result == 1) {
                System.out.println(photoId + "," + commentId + "删除成功");
            } else {
                System.out.println(photoId + "," + commentId + "删除失败");
            }

        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public void deletePhotoComment(Long photoId, String pcursor, Map<String, String> headers) {
        try {
            Map<String, Object> params = new HashMap<>();
            params.put("photoId", photoId);
            params.put("pcursor", pcursor);
            String str = HttpUtils.httpPostRequest("https://b.e.kuaishou.com/rest/kd/ad/comment/list", params, headers);
            ObjectMapper mapper = new ObjectMapper();
            mapper.configure(DeserializationFeature.FAIL_ON_UNKNOWN_PROPERTIES, false);
            JsonNode node = mapper.readTree(str);
            Integer result = node.get("result").asInt();
            pcursor = node.get("pcursor").asText();
            if (result == 1) {
                Iterator<JsonNode> nodes = node.get("rootComments").iterator();
                while (nodes.hasNext()) {
                    JsonNode commentNode = nodes.next();
                    Long commentId = commentNode.get("comment_id").asLong();
                    deleteCommentById(photoId, commentId, headers);
                }
                if (pcursor != null && !pcursor.equals("no_more")) {
                    deletePhotoComment(photoId, pcursor, headers);
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public void deleteAllComment(Map<String, String> pcursorMap) {
        try {
            Map<String, String> headers = new HashMap<>();
            Map<String, Object> params = new HashMap<>();
            headers.put("Origin", "https://b.e.kuaishou.com");
            headers.put("Referer", "https://b.e.kuaishou.com/");
            headers.put("Host", "b.e.kuaishou.com");
            headers.put("User-Agent", "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/75.0.3770.142 Safari/537.36");
            params.put("pcursorMap", pcursorMap);
            params.put("count", 30);
            String str = HttpUtils.httpPostRequest("https://b.e.kuaishou.com/rest/business/commentInfo/photos", params, headers);
            System.out.println(str);
            ObjectMapper mapper = new ObjectMapper();
            mapper.configure(DeserializationFeature.FAIL_ON_UNKNOWN_PROPERTIES, false);
            JsonNode node = mapper.readTree(str);
            Integer result = node.get("result").asInt();
            String pcursor = node.get("pcursor").asText();
            if (result == 1) {
                Iterator<JsonNode> nodes = node.get("photos").iterator();
                while (nodes.hasNext()) {
                    JsonNode photoNode = nodes.next();
                    Long photoId = photoNode.get("photoId").asLong();
                    deletePhotoComment(photoId, "", headers);
                }
                if (pcursor != null && !pcursor.equals("no_more")) {
                    pcursorMap = mapper.readValue(node.get("pcursorMap").toString(), new TypeReference<Map<String, String>>() {
                    });
                    deleteAllComment(pcursorMap);
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
    public static void main(String[] args){
        try {
            KuaishouWebInterfaceServiceImpl i = new KuaishouWebInterfaceServiceImpl();
            i.adkuaishouWebLogin("18811551872", "a123456");
            Thread.sleep(20000);
            i.deleteAllComment(new HashMap<>());
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
